@@ -3,14 +3,365 @@
 
 const app = angular.module('dataiku.notebooks.sql', ['dataiku.services', 'dataiku.filters', 'dataiku.aiSqlGeneration']);
 
+/**
+ * Directive for defining implementations of shaker hooks relevant for SQL notebook result explore view.
+ *
+ * It uses the `$scope.queryIdentifier` from the containing component, along with state params (project key and
+ * notebook id) to populate the required hook data.
+ *
+ * Note: Must be a separate directive with `scope: true` and not part of the parent component. This is required due to how
+ *       hooks are declared in the directive `shakerExploreBase`.
+ * */
+app.directive("sqlNotebookResultExploreHooksDefinition", function() {
+    return {
+        scope: true,
+        controller: function($scope, $q, $rootScope, $stateParams, ActiveProjectKey, DataikuAPI, MonoFuture) {
+            $scope.shakerWithSteps = false;
+            $scope.shakerState.filtersExplicitlyAllowed = true;
+            /* ********************* Callbacks for shakerExploreBase ******************* */
 
-app.controller('SQLNotebookController', function (
+            $scope.shakerHooks.saveForAuto = function () {
+                const deferred = $q.defer();
+                DataikuAPI.sqlNotebooks.saveExploreScript(ActiveProjectKey.get(), $stateParams.notebookId, $scope.queryIdentifier.cellId, $scope.shaker).success(function(ignored) {
+                    $scope.originalShaker = angular.copy($scope.shaker);
+                    deferred.resolve();
+                }).error(setErrorInScope.bind($scope));
+                return deferred.promise;
+            };
+
+            // We somehow miss that part of DataikuController, that is needed for shaker-in-dashboard
+            $scope.setSpinnerPosition = $scope.setSpinnerPosition || function (position) {
+                $rootScope.spinnerPosition = position;
+            };
+
+            let monoFuturizedRefresh = MonoFuture($scope).wrap(DataikuAPI.sqlNotebooks.sampleRefreshTable);
+
+            $scope.shakerHooks.getRefreshTablePromise = function (filtersOnly, filterRequest) {
+                return monoFuturizedRefresh(ActiveProjectKey.get(), $stateParams.notebookId, $scope.queryIdentifier.queryId, $scope.shaker, filterRequest);
+            };
+
+            $scope.shakerHooks.afterTableRefresh = function () {
+                // NO-OP for now
+            };
+
+            $scope.shakerHooks.updateColumnWidth = function(name, width) {
+                $scope.shaker.columnWidthsByName[name] = width;
+                $scope.autoSaveAutoRefresh();
+            };
+
+            $scope.shakerHooks.getTableChunk = function (firstRow, nbRows, firstCol, nbCols, filterRequest) {
+                return DataikuAPI.sqlNotebooks.sampleGetTableChunk(ActiveProjectKey.get(), $stateParams.notebookId, $scope.queryIdentifier.queryId, $scope.shaker,
+                    firstRow, nbRows, firstCol, nbCols, filterRequest)
+            };
+
+            $scope.shakerReadOnlyActions = true;
+            $scope.shakerWritable = false;
+            $scope.isCompareCellAvailable = false;
+
+            $scope.shakerHooks.shakerForQuery = function () {
+                return angular.copy($scope.shaker);
+            }
+
+            $scope.shakerHooks.fetchDetailedAnalysis = function (setAnalysis, handleError, columnName, alphanumMaxResults, fullSamplePartitionId, withFullSampleStatistics) {
+                // withFullSampleStatistics, fullSamplePartitionId are not relevant in this context
+                DataikuAPI.sqlNotebooks.sampleDetailedColumnAnalysis(ActiveProjectKey.get(), $stateParams.notebookId, $scope.queryIdentifier.queryId, $scope.shakerHooks.shakerForQuery(), columnName, alphanumMaxResults).success(function (data) {
+                    setAnalysis(data);
+                }).error(function (a, b, c) {
+                    if (handleError) {
+                        handleError(a, b, c);
+                    }
+                    setErrorInScope.bind($scope)(a, b, c);
+                });
+            };
+
+            $scope.canWriteProject = () => $rootScope.topNav.isProjectAnalystRW;
+
+            $scope.$watch("queryIdentifier", function(ov, nv) {
+                if (nv) {
+                    loadExploreParams();
+                }
+            }, true);
+
+            function loadExploreParams() {
+                DataikuAPI.sqlNotebooks.getExploreScript(ActiveProjectKey.get(), $stateParams.notebookId, $scope.queryIdentifier.cellId).success(function(data) {
+                    $scope.shaker = data;
+                    $scope.originalShaker = angular.copy($scope.shaker);
+                    $scope.shaker.$headerOptions = {
+                        showName: true,
+                        showMeaning: false,
+                        showDescription: false,
+                        showCustomFields: false,
+                        showProgressBar: false,
+                        showOriginalSQLTypes: true,
+                        showHeaderSeparator: true
+                    };
+                    $scope.fixupShaker();
+                    $scope.refreshTable(
+                        false,
+                        "SkipDebounceAndRunImmediately"
+                    );
+                }).error(setErrorInScope.bind($scope));
+            }
+        }
+    }
+});
+
+app.component("sqlNotebookResultExplore", {
+    templateUrl: "/templates/notebooks/sql-notebook-result-explore.html",
+    bindings: {
+        isRunning: '<',
+        cellId: '<',
+        queryId: '<'
+    },
+    controller: function($scope, translate) {
+        const $ctrl = this;
+        $scope.translate = translate;
+        // Required for detailed analysis of alphanumeric columns
+        $scope.sanitize = sanitize;
+
+        $ctrl.$onChanges = () => {
+            if (!$ctrl.isRunning) { // Only apply changes when query is done running
+                $scope.queryIdentifier = {queryId: $ctrl.queryId, cellId: $ctrl.cellId};
+            }
+        };
+    }
+});
+
+app.constant("SQL_NOTEBOOK_CHART_MODE", {
+    CREATE: "create",
+    EDIT: "edit"
+});
+
+app.controller('SqlNotebookChartEditorController', function ($scope, $rootScope, $controller, SqlNotebookResultDataFetcher, DashboardPageUtils, DSSVisualizationThemeUtils, ChartTypeChangeHandler, SQL_NOTEBOOK_CHART_MODE) {
+    $controller('ShakerChartsCommonController', {$scope: $scope});
+    $scope.isSqlNotebook = true;
+    $scope.SQL_NOTEBOOK_CHART_MODE = SQL_NOTEBOOK_CHART_MODE;
+
+    $scope.getDefaultNewChart = function() {
+        const defaultTheme = DSSVisualizationThemeUtils.getThemeOrDefault($rootScope.appConfig.selectedDSSVisualizationTheme);
+        return {
+            theme: defaultTheme,
+            def: ChartTypeChangeHandler.defaultNewChart(defaultTheme)
+        };
+    };
+
+    $scope.overrideFormattingWithTheme = function(theme) {
+        const currentChartCopy = angular.copy($scope.chart);
+        $scope.chart.theme = theme;
+        DSSVisualizationThemeUtils.applyToChart({ chart: $scope.chart.def, theme, formerTheme: $scope.chart.theme });
+        DSSVisualizationThemeUtils.showThemeAppliedSnackbar($scope.chart, currentChartCopy);
+    };
+
+    $scope.$on('$destroy', () => DSSVisualizationThemeUtils.hideThemeAppliedSnackbar());
+
+    $scope.validity = {}; // Needed by chart redraw
+    $scope.getDataSpec = SqlNotebookResultDataFetcher.getDataSpec;
+
+    $scope.saveChart = function() {
+        // This editor is only to define the chart, saving is delegated to after modal resolution.
+        // Therefore, there is no need to save here.
+    };
+
+    $scope.fixExpression = (expression, fixName = "plus") => {
+        return SqlNotebookResultDataFetcher.fixExpression(expression, fixName, $scope.queryId);
+    };
+
+    $scope.getExecutePromise = function(request, saveShaker = false, noSpinner = true, requiredSampleId = undefined, dataSpec = $scope.getDataSpec()) {
+        return SqlNotebookResultDataFetcher.getPivotResponse(request, $scope.queryId);
+    };
+
+    $scope.fetchColumnsSummary = function () {
+        return SqlNotebookResultDataFetcher.getColumnSummary($scope.queryId).then(function (columns) {
+            $scope.chart.summary = columns.data;
+            $scope.makeUsableColumns($scope.chart.summary);
+        })
+    };
+
+    /** We are a bit stricter than just valid for creating/editing charts, as we only ever want to allow creation
+     * of charts that can be rendered. Otherwise, user needs to make adjustments in the modal.
+     *  */
+    $scope.canConfirm = function() {
+        return $scope.validity && $scope.validity.valid && !$scope.allFilteredOut && !$scope.display0Warning;
+    };
+
+    // There will be at most one chart in this editor
+    $scope.charts = [];
+    if ($scope.mode === SQL_NOTEBOOK_CHART_MODE.CREATE) {
+        // This weird code will create a default chart and add it to charts
+        $scope.addChart();
+        $scope.chart = $scope.charts[$scope.currentChart.index];
+        $scope.chart.customMeasures = [];
+        $scope.chart.reusableDimensions = [];
+        $scope.chart.hierarchies = [];
+    } else {
+        $scope.chart = $scope.chartToEdit;
+        $scope.addChart({chart: $scope.chart});
+    }
+
+    $scope.validateChart = () => {
+        $scope.resolveModal($scope.chart);
+    };
+    $scope.cancel = () => {
+        $scope.dismiss();
+    };
+
+    // chartHandler options
+    $scope.noThumbnail = true;
+    $scope.addCustomMeasuresToScopeAndCache($scope.chart.customMeasures);
+    $scope.addBinnedDimensionToScopeAndCache($scope.chart.reusableDimensions);
+    $scope.addHierarchiesToScopeAndCache($scope.chart.hierarchies);
+});
+
+app.factory("SqlNotebookChartEditor", function($rootScope, Logger, CreateModalFromTemplate, SQL_NOTEBOOK_CHART_MODE) {
+    function createOrEditChart(queryId, mode, chartToEdit, postConfirmCallBack) {
+        const newScope = $rootScope.$new();
+        newScope.mode = mode;
+        newScope.queryId = queryId;
+        if (chartToEdit) {
+            newScope.chartToEdit = angular.copy(chartToEdit);
+        }
+        CreateModalFromTemplate("/templates/notebooks/sql-notebooks-new-chart-modal.html", newScope, 'SqlNotebookChartEditorController', undefined, true).then(function(chart) {
+            Logger.info("Chart edition confirmed");
+            postConfirmCallBack(chart);
+        }).catch(() => {
+            Logger.info("Chart edition dismissed");
+        });
+    }
+
+    return {
+        createChart: function(queryId, postCreationCallback) {
+            createOrEditChart(queryId, SQL_NOTEBOOK_CHART_MODE.CREATE, null, postCreationCallback);
+        },
+        editChart: function(queryId, chartToEdit, postEditionCallback) {
+            createOrEditChart(queryId, SQL_NOTEBOOK_CHART_MODE.EDIT, chartToEdit, postEditionCallback)
+        }
+    };
+});
+
+app.factory("SqlNotebookResultDataFetcher", function(ActiveProjectKey, DataikuAPI, $stateParams) {
+    return {
+        getColumnSummary: function(queryId) {
+            return DataikuAPI.sqlNotebooks.getColumnSummary(ActiveProjectKey.get(), $stateParams.notebookId, queryId);
+        },
+        getPivotResponse: function(request, queryId) {
+            return DataikuAPI.sqlNotebooks.getPivotResponse(request, ActiveProjectKey.get(), $stateParams.notebookId, queryId);
+        },
+        fixExpression: function(expression, fixName, queryId) {
+            return DataikuAPI.sqlNotebooks.fixExpression(ActiveProjectKey.get(), $stateParams.notebookId, queryId, expression, fixName);
+        },
+        // This is not per se a data callback, but used in multiple places, so it is upgraded as a first class citizen
+        getDataSpec: function() {
+            return {
+                datasetProjectKey: $stateParams.projectKey,
+                datasetName: "result_from_sql_query"
+            };
+        }
+    }
+});
+
+app.component("sqlNotebookResultChart", {
+    templateUrl: "/templates/notebooks/sql-notebook-result-chart.html",
+    bindings: {
+        chart: '<',
+        queryId: "<"
+    },
+    controller: function($scope, $element, $controller, ChartSetErrorInScope, ChartDefinitionChangeHandler, SqlNotebookResultDataFetcher, Logger, MonoFuture, ChartRequestComputer, translate) {
+        const $ctrl = this;
+        $controller('ShakerChartsCommonController', { $scope: $scope });
+        $scope.initChartCommonScopeConfig($scope);
+        ChartSetErrorInScope.defineInScope($scope);
+
+        $scope.translate = translate;
+
+        $scope.getDataSpec = SqlNotebookResultDataFetcher.getDataSpec;
+
+        function fetchColumnsSummary() {
+            return SqlNotebookResultDataFetcher.getColumnSummary($ctrl.queryId).then(function (columns) {
+                $scope.chart.summary = columns.data;
+                $scope.makeUsableColumns($scope.chart.summary);
+            });
+        }
+
+        $scope.getExecutePromise = function(request) {
+            return SqlNotebookResultDataFetcher.getPivotResponse(request, $ctrl.queryId);
+        };
+
+        const executePivotRequest = MonoFuture($scope).wrap($scope.getExecutePromise);
+        $scope.chartSpecific = {};
+        $scope.noThumbnail = true;
+
+        function setupAndLoadResults() {
+            Logger.info("Force re-draw chart");
+            $scope.chart = $ctrl.chart;
+            fetchColumnsSummary().then(function() {
+                const request = ChartRequestComputer.compute($scope.chart.def, $element.width(), $element.height(), $scope.chartSpecific);
+                executePivotRequest(request).success(data => {
+                    Logger.info('Got pivot response for chart');
+                    resetErrorInScope($scope);
+                    $scope.response = data;
+                    $scope.setValidity({valid: true});
+                }).error(function(data, headers, status) {
+                    $scope.setValidity({valid: false});
+                    $scope.chartSetErrorInScope(data, headers, status);
+                });
+            });
+        }
+
+        // Minimal handling of resizing, triggered when resizer is done moving
+        $scope.$on("reflow", function() {
+            $scope.$broadcast("redraw");
+        });
+
+        function themeChanged(newChart, oldChart) {
+            if (!newChart ||  !newChart.theme) {
+                return false;
+            }
+            if (!oldChart || !oldChart.theme) {
+                return true;
+            }
+            return !angular.equals(newChart.theme, oldChart.theme);
+        }
+
+        // Change logic to trigger change in chart or redraw
+        let oldChart, oldQueryId;
+        $ctrl.$doCheck = () => {
+            // First hit or when changing query, we know we want to redraw
+            if (!oldChart || oldQueryId !== $ctrl.queryId) {
+                oldChart = angular.copy($ctrl.chart);
+                oldQueryId = $ctrl.queryId;
+                setupAndLoadResults();
+                return;
+            }
+            // Otherwise, we make sure it is worth re-drawing
+            if (!angular.equals(oldChart, $ctrl.chart)) {
+                const importantChangedDefinition = ChartDefinitionChangeHandler.getRecomputeChange($ctrl.chart.def, oldChart.def);
+                const changedTheme = themeChanged($ctrl.chart, oldChart);
+                oldChart = angular.copy($ctrl.chart);
+                oldQueryId = $ctrl.queryId;
+                if (importantChangedDefinition || changedTheme) {
+                    setupAndLoadResults();
+                }
+            }
+        };
+
+    }
+});
+
+app.controller('SQLNotebookControllerCM6', function (
         $scope, $state, $timeout, $q, $modal, $stateParams, $rootScope,
         Assert, WT1, Logger, DataikuAPI, Dialogs, TopNav, BigDataService,
         LocalStorage, SQLExplorationService, CreateExportModal,
-        ExportUtils, CreateModalFromTemplate, NotebooksUtils, MonoFuture, AISqlGenerationService, RatingFeedbackParams, ClipboardUtils) {
-
+        ExportUtils, CreateModalFromTemplate, NotebooksUtils, MonoFuture, AISqlGenerationService, RatingFeedbackParams, ClipboardUtils,
+        Ng2SQLNotebookStore
+) {
 	/* *********************** Basic CRUD ********************* */
+    $scope.initSQLNotebookStore = function() {
+        Ng2SQLNotebookStore.initNotebook($scope.notebookParams, $scope.connectionDetails);
+    }
+
+    $scope.clearSQLNotebookStore = function() {
+        Ng2SQLNotebookStore.clearNotebook();
+    }
 
     $scope.loadNotebook = function() {
     	return DataikuAPI.sqlNotebooks.getSummary($stateParams.projectKey, $stateParams.notebookId).then(({data}) => {
@@ -34,6 +385,7 @@ app.controller('SQLNotebookController', function (
             for (var i = 0; i < connections.nconns.length; i++) {
                 if (connections.nconns[i].name == $scope.notebookParams.connection) {
                     $scope.connectionDetails = connections.nconns[i];
+                    $scope.initSQLNotebookStore();
                     break;
                 }
             }
@@ -45,7 +397,6 @@ app.controller('SQLNotebookController', function (
 
             $scope.notebookLocalState = {};
             $scope.notebookTmpState = {};
-            $scope.notebookLocalState.cellMode = 'SINGLE';
             $scope.notebookLocalState.tableListingMode = 'PROJECT';
             $scope.notebookLocalState.tableOrdering = 'TABLE';
             $scope.notebookLocalState.leftPaneTab = 'Cells';
@@ -155,6 +506,7 @@ app.controller('SQLNotebookController', function (
                 }, 500);
             }
         });
+        loadChartParamsIfNeeded();
         loadResultsIfNeeded();
     };
 
@@ -384,6 +736,8 @@ app.controller('SQLNotebookController', function (
             var selectedCell = $scope.selectedCell();
             if (!selectedCell) {
                 $scope.selectCell(0);
+            } else { // This is required for first cell selection coming from local state
+                loadChartParamsIfNeeded();
             }
         }
     };
@@ -400,6 +754,7 @@ app.controller('SQLNotebookController', function (
             });
         }
         $scope.notebookLocalState = $.extend($scope.notebookLocalState || {}, localState.notebookLocalState);
+        cleanLocalState();
         if (!$scope.aiGenerateSQLEnabled && $scope.notebookLocalState.leftPaneTab === 'Generate') {
             $scope.notebookLocalState.leftPaneTab = 'Tables';
         }
@@ -408,6 +763,11 @@ app.controller('SQLNotebookController', function (
         });
         onLocalStatesLoaded();
     };
+    
+    function cleanLocalState() {
+        // Cell mode is deprecated and no longer used. We remove it to avoid confusion.
+        delete $scope.notebookLocalState.cellMode;
+    }
 
     var doSaveLocalStates = function() {
         if (!$scope.notebookParams) return;
@@ -432,21 +792,16 @@ app.controller('SQLNotebookController', function (
 
     /* ************  Text insertion *************** */
 
-    function insertText(text, addSelectIfEmptyQuery) {
-        var cell = $scope.selectedCell();
-        if (cell && cell.$tmpState.insertText) {
-            cell.$tmpState.insertText(text, addSelectIfEmptyQuery);
-        } else {
-            Logger.warn("Cannot insert text: no current cell or no insertText function");
-        }
+    $scope.insertText = function (text, addSelectIfEmptyQuery) {
+        Ng2SQLNotebookStore.insertText(text, addSelectIfEmptyQuery);
     }
 
     $scope.onTableClicked = function(table) {
-        insertText(table.quoted, true);
+        $scope.insertText(table.quoted, true);
     };
 
     $scope.onFieldClicked = function(field) {
-        insertText(field.quotedName);
+        $scope.insertText(field.quotedName);
     };
 
     $scope.onTableExplorerError = function(args) {
@@ -454,7 +809,7 @@ app.controller('SQLNotebookController', function (
     }
 
     $scope.insertCodeSnippet = function(snippet) {
-        insertText(snippet.code);
+        $scope.insertText(snippet.code);
     }
 
     /* ************ History ************ */
@@ -482,13 +837,22 @@ app.controller('SQLNotebookController', function (
 
     var loadResultsIfNeeded = function() {
         var cell = $scope.selectedCell();
-        if ($scope.notebookLocalState.cellMode == 'SINGLE' && cell.$localState.selected) {
+        if (cell.$localState.selected) {
             if (!$scope.fetchingResults[cell.id] && cell.$tmpState.lastQuery  && !cell.$tmpState.runningQuery && !cell.$tmpState.results) { //&& cell.$tmpState.lastQuery.state == 'DONE'
                 $scope.fetchLastResults(cell, cell.$tmpState.lastQuery.id)
             }
         }
     };
 
+    function loadChartParamsIfNeeded() {
+        const cell = $scope.selectedCell();
+        if (!cell.$localExploreChart) {
+            DataikuAPI.sqlNotebooks.getExploreChart($stateParams.projectKey, $stateParams.notebookId, cell.id).success(function(data) {
+                cell.$localExploreChart = data;
+            }).error(setErrorInScope.bind($scope));
+        }
+    }
+
     $scope.loadQuery = function(cell, query, fetchResults) {
         Assert.trueish(cell, 'no cell');
         delete cell.$tmpState.results;
@@ -542,12 +906,42 @@ app.controller('SQLNotebookController', function (
     $scope.ratingFeedbackParams.showRatingFeedback = false;
     $scope.ratingFeedbackParams.requestId = null;
     $scope.ratingFeedbackParams.featureRated = "text2sql";
+
+    $scope.$on("$destroy", () => {
+        $scope.clearSQLNotebookStore();
+    });
+
+});
+
+/**
+ * Controller to be removed once we remove feature flag dku.feature.newSQLNotebookEditor
+ * isolates all overrides to new sql editor controller that are necessary for legacy editor to work
+ */
+app.controller('SQLNotebookControllerCM5', function ($scope, $controller, Logger) {
+    $controller('SQLNotebookControllerCM6', {$scope: $scope});
+
+    $scope.insertText = function(text, addSelectIfEmptyQuery) {
+        var cell = $scope.selectedCell();
+        if (cell && cell.$tmpState.insertText) {
+            cell.$tmpState.insertText(text, addSelectIfEmptyQuery);
+        } else {
+            Logger.warn("Cannot insert text: no current cell or no insertText function");
+        }
+    }
+
+    $scope.initSQLNotebookStore = function () {
+        // NO-OP
+    }
+
+    $scope.clearSQLNotebookStore = function() {
+        // NO-OP
+    }
+
 });
 
 
-
-app.controller('SqlNotebookQueryCellController', function ($scope, $element, $stateParams, $timeout, $q, DataikuAPI, WT1, Logger,
-               StateUtils, Dialogs, CreateModalFromTemplate, CreateExportModal, BigDataService, SQLExplorationService, ExportUtils, CodeMirrorSettingService) {
+app.controller('SqlNotebookQueryCellController', function ($scope, $element, $stateParams, $timeout, $q, DataikuAPI, WT1, Logger, $rootScope,
+               StateUtils, Dialogs, CreateModalFromTemplate, CreateExportModal, BigDataService, SQLExplorationService, ExportUtils, CodeMirrorSettingService, DKUSQLFormatter, Ng2SQLNotebookStore, SpinnerService) {
 
     /* ************  Execution ************ */
 
@@ -696,9 +1090,24 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
     };
 
     /* ************ Code edition ************ */
+    // Provide CM5-like API while we are in CM6 world.
+    // We are doing this to support both editors with the same code base.
+    // TODO remove once we remove feature flag dku.feature.newSQLNotebookEditor.enabled
+    $scope.cm = {
+        getSelection() {
+            return $scope.codeSelectionContent;
+        },
+        focus() {
+            Ng2SQLNotebookStore.focusEditor();
+        },
+        // NO-OPs
+        setOption() {},
+        on() {}
+    }
 
     $scope.autocompleteSQL = function(cm,type) {
-        SQLExplorationService.listTables($scope.notebookParams.connection, $stateParams.projectKey).then(function(tables) {
+        SpinnerService.lockOnPromise(
+            SQLExplorationService.listTables($scope.notebookParams.connection, $stateParams.projectKey).then(function(tables) {
             var fieldsToAutocomplete = CodeMirror.sqlFieldsAutocomplete(cm, tables);
             if (fieldsToAutocomplete && fieldsToAutocomplete.length) {
                 SQLExplorationService.listFields($scope.notebookParams.connection, fieldsToAutocomplete).then(function(data) {
@@ -711,9 +1120,11 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
                     return CodeMirror.sqlNotebookHint(editor, type+"-notebook", tables.map(function(t) {return t.table;}), null);
                 }, {completeSingle:false});
             }
-        });
+        }));
     };
 
+    // TODO: remove when we end support for CM5
+    // Triggered within CM5 insertText()
     $scope.cell.$tmpState.insertText = function(text, addSelectIfEmptyQuery) {
         if (addSelectIfEmptyQuery && $scope.isQueryEmpty()) {
             text = 'SELECT * FROM '+text;
@@ -841,29 +1252,6 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
 
     /* ************ UI ************ */
 
-    var setupFattable = function(results) {
-        if(!results) {
-            return;
-        }
-        $scope.columnWidths = [];
-        for(let k in results.columns) {
-            var name = results.columns[k].name;
-            var size = 100;
-            if(name) {
-                size = Math.max(size,name.length*7+10);
-            }
-            $scope.columnWidths.push(size);
-        }
-        for(var r in results.rows) {
-            for(let k in results.rows[r]) {
-                var cell = results.rows[r][k];
-                if(cell) {
-                    $scope.columnWidths[k] = Math.max(cell.length*7+10,$scope.columnWidths[k]);
-                }
-            }
-        }
-    };
-
     var setupAutoComplete = function(tables, suggestions) { //TODO tables should be in suggestions?
         if (!$scope.cm) {
             Logger.warn("Failed to setup autocomplete");
@@ -881,6 +1269,7 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
 
     $scope.focusQuery = function() {
         if (!$scope.cm) return;
+        // TODO: remove when we end support for CM5
         $scope.cm.focus();
     };
 
@@ -913,6 +1302,7 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
             sql : ''
         };
         if ($scope.cm && $scope.notebookMode == 'HIVE' || $scope.notebookMode == 'IMPALA' || $scope.notebookMode == 'SPARKSQL') {
+            // TODO: remove when we end support for CM5
             $scope.cm.setOption('mode', 'text/x-hivesql');
         }
     }
@@ -921,8 +1311,25 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
     $scope.$on('autocompleteSuggestionsLoaded', setupAutoComplete);
 
     function selectionChange() {
+        // TODO: remove when we end support for CM5
         $scope.somethingSelected = $scope.cm && $scope.cm.somethingSelected() && $scope.cm.getSelection().trim();
         $timeout(() => $scope.$apply());
+    }
+    $scope.codeChange = function(code) {
+        // Because the angular model() is not properly triggering AngularJS digest cycle
+        $scope.$applyAsync(() => {
+            $scope.cell.$localState.query.sql = code;
+        });
+    }
+    $scope.selectionChange = function ({ from, to }) {
+        const newContent = $scope.cell?.$localState?.query?.sql?.slice(from, to);
+        if ($scope.codeSelectionContent === newContent) {
+            return;
+        }
+        $scope.$applyAsync(() => {
+            $scope.somethingSelected = from != null && to != null && from < to;
+            $scope.codeSelectionContent = newContent;
+        });
     };
 
     $scope.editorOptions = function() {
@@ -931,6 +1338,7 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
             noFullScreen: true,
             onLoad: function(cm) {
                 $scope.cm = cm;
+                // TODO: remove when we end support for CM5
                 $scope.cm.on("cursorActivity", selectionChange);
                 if ($scope.notebookMode) {
                     setTimeout(function () {
@@ -961,20 +1369,26 @@ app.controller('SqlNotebookQueryCellController', function ($scope, $element, $st
     }
     $scope.$watch('cell.$localState.query.sql', saveLocalStateLater);
 
-    $scope.$watch('cell.$tmpState.results', setupFattable ,false);
-
     $scope.$watch('cell.$tmpState.resultsTab', function(nv) {
         if (nv != 'logs') {
             $scope.reflow();
         }
     });
 
-    setupFattable($scope.cell.$tmpState.results); //DEBUG
-
     $scope.reflow = function() {
         $scope.$broadcast("reflow"); // update fat repeat layout
     }
 
+    $scope.formatQuery = () => {
+        const query = $scope.cell?.$localState?.query?.sql;
+        if (query) {
+            const editorSettings = $rootScope.appConfig.userSettings.codeEditor;
+            const cellLocalStateCopy = angular.copy($scope.cell.$localState);
+            cellLocalStateCopy.query.sql = DKUSQLFormatter.format(query, editorSettings.indentUnit, editorSettings.indentWithTabs);
+            $scope.cell.$localState = cellLocalStateCopy;
+        }
+    };
+
     var initLastQuery = function() {
         // Check if the current code is the same as last query, in this case mark the last query as active and load results
         const cellHistory = $scope.notebookTmpState.history[$scope.cell.id] || [];
@@ -1067,26 +1481,108 @@ app.directive('sqlNotebookQueryCell', function($stateParams, DataikuAPI){
 });
 
 
-app.directive('sqlNotebookQuerySingleCell', function(){
+app.directive('sqlNotebookQuerySingleCell', function (ActiveProjectKey, CreateModalFromTemplate, SqlNotebookChartEditor, Logger, Dialogs, DataikuAPI, $stateParams) {
     return {
-        templateUrl:'/templates/notebooks/sql-notebook-query-single-cell.html',
+        templateUrl: '/templates/notebooks/sql-notebook-query-single-cell.html',
         controller: 'SqlNotebookQueryCellController',
-        link : function(scope, element, attrs) {
-            scope.cell.$tmpState.error = function(a,b,c){
+        link: function (scope, element, attrs) {
+            scope.cell.$localState.query = scope.cell.$localState.query || {};
+
+            scope.cell.$tmpState.error = function (a, b, c) {
                 if ($('.local-api-error', element).length > 0) {
-                    setErrorInScope.bind($('.local-api-error', element).scope())(a,b,c);
+                    setErrorInScope.bind($('.local-api-error', element).scope())(a, b, c);
                 }
             };
-            scope.cell.$tmpState.clearError = function(){
+            scope.cell.$tmpState.clearError = function () {
                 if ($('.local-api-error', element).length > 0) {
                     resetErrorInScope($('.local-api-error', element).scope());
                 }
             };
+
+            scope.saveChartData = function() {
+                DataikuAPI.sqlNotebooks.saveExploreChart(ActiveProjectKey.get(), $stateParams.notebookId, scope.cell.id, scope.cell.$localExploreChart).success(function(ignored) {
+                    // Nothing to do
+                }).error(setErrorInScope.bind(scope));
+            }
+
+            scope.onSelectChart = (chartId) => {
+                scope.cell.$localState.activeTab = {id: chartId, type: 'chart'};
+                scope.cell.$selectedChart = getChart(chartId);
+            };
+
+            scope.selectTableTab = () => {
+                scope.cell.$localState.activeTab = {name: 'Table', type: 'table'}
+            };
+
+            scope.onCreateChart = () => {
+                SqlNotebookChartEditor.createChart(scope.cell.$localState.query.id, (createdChart) => {
+                    Logger.info("Chart created");
+                    scope.cell.$localExploreChart.charts.push(createdChart);
+                    scope.saveChartData();
+                    scope.onSelectChart(createdChart.def.id);
+                });
+            };
+
+            function getChart(chartId) {
+                return scope.cell.$localExploreChart.charts.find(chart => chart.def.id === chartId);
+            }
+
+            scope.onEditChart = (chartIdToEdit) => {
+                const chartToEdit = getChart(chartIdToEdit);
+                SqlNotebookChartEditor.editChart(scope.cell.$localState.query.id, chartToEdit, (editedChart) => {
+                    Logger.info("Chart edited");
+                    updateChart(chartToEdit, editedChart);
+                    scope.saveChartData();
+                    scope.onSelectChart(editedChart.def.id);
+                });
+            };
+
+            scope.onDeleteChart = (deletedChartId) => {
+                if (!scope.cell.$localExploreChart) {
+                    return;
+                }
+                Dialogs.confirm(scope, "Confirm deletion", "Are you sure you want to remove this chart ?").then(function() {
+                    scope.cell.$localExploreChart.charts = scope.cell.$localExploreChart.charts.filter(chart => chart.def.id !== deletedChartId);
+                    scope.saveChartData();
+                    if (scope.cell.$localState.activeTab.id === deletedChartId) {
+                        scope.selectTableTab();
+                    }
+                }, () => {});
+            };
+
+            function updateChart(baseChart, updatedChart) {
+                scope.cell.$localExploreChart.charts = scope.cell.$localExploreChart.charts.map(chart => {
+                    if (chart.def.id === baseChart.def.id) {
+                        return updatedChart;
+                    }
+                    return chart;
+                })
+            }
+
+            // Counter for duplicated chart names
+            scope.$watch('cell.$localExploreChart', function (nv) {
+                if (!nv) {
+                    return;
+                }
+                const counts = {};
+                for (const item of scope.cell.$localExploreChart.charts) {
+                    counts[item.def.name] = (counts[item.def.name] || 0) + 1;
+                }
+                // An object to keep track of the current suffix index for each item.
+                const trackers = {};
+
+                scope.cell.$localExploreChart.chartNames = scope.cell.$localExploreChart.charts.map(chart => {
+                    if (counts[chart.def.name] > 1) {
+                        trackers[chart.def.name] = (trackers[chart.def.name] || 0) + 1;
+                        return `${chart.def.name} (${trackers[chart.def.name]})`;
+                    }
+                    return chart.def.name;
+                })
+            }, true);
         }
     }
 });
 
-
 app.directive('localApiError', function(){
     return {
         scope: {}
@@ -1124,7 +1620,7 @@ app.directive('sqlNotebookMdSingleCell', function(DataikuAPI, $stateParams){
     };
 });
 
-app.directive('sqlTableExplorer', function($stateParams, SQLExplorationService, Debounce, Logger, ConnectionExplorationService, translate) {
+app.directive('sqlTableExplorer', function($stateParams, SQLExplorationService, Debounce, Logger, ConnectionExplorationService, translate, SpinnerService) {
     function generateOnlyLastCall() {
         var ref = null;
         return function() {
@@ -1227,15 +1723,12 @@ app.directive('sqlTableExplorer', function($stateParams, SQLExplorationService,
 
                 scope.fatList = []; // purge previous tables
 
-                return (scope.notebook.tableListingMode === 'PROJECT'
+                return SpinnerService.lockOnPromise((scope.notebook.tableListingMode === 'PROJECT'
                     ? SQLExplorationService.listTablesFromProject(scope.connection, $stateParams.projectKey)
                     : SQLExplorationService.listTables(scope.connection, $stateParams.projectKey, chosenCatalog, chosenSchema))
                     .then((tables) => {
                         handleTables(tables);
                     })
-                    .catch((error) => {
-                        scope.reportErrorFn(error);
-                    }
                 )
             }
 
