@@ -1083,6 +1083,9 @@ app.controller("ProjectSettingsExposedController", function($rootScope, $scope,
             case 'MODEL_EVALUATION_STORE':
                 $state.go('projects.project.modelevaluationstores.modelevaluationstore.evaluations',{mesId: object.localName});
                 break;
+            case 'GENAI_EVALUATION_STORE':
+                $state.go('projects.project.genaievaluationstores.genaievaluationstore.evaluations',{mesId: object.localName});
+                break;
             case 'MANAGED_FOLDER':
                 $state.go('projects.project.managedfolders.managedfolder.view',{odbId: object.localName});
                 break;
@@ -1098,6 +1101,12 @@ app.controller("ProjectSettingsExposedController", function($rootScope, $scope,
             case 'SCENARIO':
                 $state.go('projects.project.scenarios.scenario.steps',{scenarioId: object.localName});
                 break;
+            case 'AGENT_TOOL':
+                $state.go('projects.project.agenttools.agenttool',{agentToolId: object.localName});
+                break;
+            case 'RETRIEVABLE_KNOWLEDGE':
+                $state.go('projects.project.retrievableknowledges.retrievableknowledge.usage',{retrievableKnowledgeId: object.localName});
+                break;
         }
     };
 
