@@ -238,6 +238,7 @@ class StreamingAPI(AbstractStreamingAPI):
             self.activity_id = ""
         else:
             self.activity_id = flow.FLOW.get("currentActivityId", "") # will be missing in CAK
+        self.done_event = threading.Event()
 
     def init_session(self, request):
         """
@@ -288,7 +289,16 @@ class StreamingAPI(AbstractStreamingAPI):
 
         :param string id: identifier of a writing session, as returned by :meth:`init_session()`
         """
-        decoded_resp = jek_or_backend_get_call("datasets/wait-write-session/", params={"id": id})
+        def heartbeat():
+            while True:
+                logging.info("Waiting for end ...")
+                if self.done_event.wait(10):
+                    logging.info("Got done, exiting")
+                    break
+                else:
+                    logging.info("Not done yet")
+                    yield (str(time.time()) + "\n").encode("utf-8")  # generator myst yield bytes
+        decoded_resp = jek_or_backend_get_call("datasets/wait-write-session/", params={"id": id}, data=heartbeat())
         if decoded_resp["ok"]:
             writtenRows = decoded_resp["writtenRows"]
             print ("%s rows successfully written (%s)" % (writtenRows,id))
@@ -304,6 +314,8 @@ class StreamingAPI(AbstractStreamingAPI):
         jek_or_backend_void_call("datasets/push-data/", params={"id": id}, data=generator, err_msg="Streaming: push-data call failed")
         # We don't really care about whether this call failed or not.
 
+    def close(self):
+        self.done_event.set()
 
 # Create a thread which is waiting for the streaming session to complete.
 class WriteSessionWaiter(threading.Thread):
@@ -340,6 +352,7 @@ class WriteSessionWaiter(threading.Thread):
         """
         Wait for the writing session to finish, raise if needed.
         """
+        self.streaming_api.close()
         self.join()
         self.raise_on_failure()
 
