@@ -219,14 +219,13 @@ class ClassicalSearchRunner(TabularSearchRunner):
 class TimeseriesForecastingSearchRunner(TabularSearchRunner):
 
     def __init__(
-            self, trainable_model, search_settings, model_folder_context, split_handler, fit_before_predict,
+            self, trainable_model, search_settings, model_folder_context, split_handler,
             min_timeseries_size_for_training, evaluation_metric, metric_sign, skip_too_short_timeseries,
             custom_train_test_intervals):
         super(TimeseriesForecastingSearchRunner, self).__init__(trainable_model=trainable_model,
                                                                 search_settings=search_settings,
                                                                 model_folder_context=model_folder_context,
                                                                 evaluation_metric=evaluation_metric)
-        self.fit_before_predict = fit_before_predict
         self.min_timeseries_size_for_training = min_timeseries_size_for_training
         self.split_handler = split_handler
         self.model_scorer = None
@@ -246,7 +245,7 @@ class TimeseriesForecastingSearchRunner(TabularSearchRunner):
     def initialize_search_context(self, df):
         if not self.search_skipped():
             self.search_context = TimeseriesForecastingSearchContext(
-                df, self.split_handler, self.model_scorer, self.preprocessed_external_features, self.shift_map, self.fit_before_predict,
+                df, self.split_handler, self.model_scorer, self.preprocessed_external_features, self.shift_map,
                 self.min_timeseries_size_for_training, self.evaluation_metric, self.metric_sign, self._trainable_model,
                 self.skip_too_short_timeseries, self.custom_train_test_intervals
             )
