@@ -735,7 +735,7 @@ RUN groupadd -r dataiku \
     def copy_kit(self):
         # setup pod install dir
         self.dockerfile += """
-RUN mkdir -p {podinstalldir}
+RUN mkdir -p {podinstalldir} && chown -Rh dataiku:dataiku {podinstalldir}
 """.format(podinstalldir=POD_DKUINSTALLDIR)
 
         # see make-studio-package.sh
@@ -842,7 +842,7 @@ RUN mkdir -p {podinstalldir}
         else:
             for item in ["dist", "dss-version.json", "dss-arch.txt", "installer.sh",
                          "lib", "python", PYTHON_PACKAGES, "R", "resources", "scripts", "conda.packages"]:
-                self.dockerfile += 'COPY "%s" %s/%s\n' % (item, POD_DKUINSTALLDIR, item)
+                self.dockerfile += 'COPY --chown=dataiku:dataiku "%s" %s/%s\n' % (item, POD_DKUINSTALLDIR, item)
                 self.copy_to_buildenv(osp.join(DKUINSTALLDIR, item), item)
 
             # Remove some files that are not actually useful and that trigger stupid security scanners
@@ -856,8 +856,7 @@ RUN rm -f {podinstalldir}/resources/code-envs/python/image-classification/v1/spe
     && rm -f {podinstalldir}/resources/code-envs/python/object-detection/v1/spec/requirements.txt \
     && rm -rf {podinstalldir}/lib/ivy/govern-server-run \
     && rm -f {podinstalldir}/lib/ivy/backend-run/jnr-posix-3.0.44.jar \
-    && rm -rf {podinstalldir}/python36.packages/nbconvert* \
-    && chown -Rh dataiku:dataiku {podinstalldir}
+    && rm -rf {podinstalldir}/python36.packages/nbconvert* 
 """.format(podinstalldir=POD_DKUINSTALLDIR)
 
 class APIImageBuilder(KitImageBuilder):
